'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _soundcloudAudio = require('soundcloud-audio');

var _soundcloudAudio2 = _interopRequireDefault(_soundcloudAudio);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _audioStore = require('../utils/audioStore.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SoundPlayerContainer = function (_Component) {
    _inherits(SoundPlayerContainer, _Component);

    function SoundPlayerContainer(props, context) {
        _classCallCheck(this, SoundPlayerContainer);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(SoundPlayerContainer).call(this, props, context));

        if (!props.clientId && !props.soundCloudAudio) {
            throw new Error('You need to get a clientId from SoundCloud,\n                or pass in an instance of SoundCloudAudio.\n                https://github.com/soundblogs/react-soundplayer#usage');
        }

        // Don't create a SoundCloudAudio instance
        // if there is no `window`
        if ('undefined' !== typeof window) {
            if (props.soundCloudAudio) {
                _this.soundCloudAudio = props.soundCloudAudio;
            } else {
                _this.soundCloudAudio = new _soundcloudAudio2.default(props.clientId);
            }
        }

        _this.state = {
            duration: 0,
            currentTime: 0,
            seeking: false,
            playing: false
        };
        _this.wrapChild = _this.wrapChild.bind(_this);
        return _this;
    }

    _createClass(SoundPlayerContainer, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            var _this2 = this;

            var soundCloudAudio = this.soundCloudAudio;
            var _props = this.props;
            var resolveUrl = _props.resolveUrl;
            var streamUrl = _props.streamUrl;


            if (streamUrl) {
                soundCloudAudio.preload(streamUrl);
            } else if (resolveUrl) {
                soundCloudAudio.resolve(resolveUrl, function (data) {
                    _this2.setState(_defineProperty({}, data.tracks ? 'playlist' : 'track', data));
                });
            }

            // https://developer.mozilla.org/en-US/docs/Web/Guide/Events/Media_events
            soundCloudAudio.on('playing', this.onAudioStarted.bind(this));
            soundCloudAudio.on('timeupdate', this.getCurrentTime.bind(this));
            soundCloudAudio.on('loadedmetadata', this.getDuration.bind(this));
            soundCloudAudio.on('seeking', this.onSeekingTrack.bind(this));
            soundCloudAudio.on('seeked', this.onSeekedTrack.bind(this));
            soundCloudAudio.on('pause', this.onAudioPaused.bind(this));
            soundCloudAudio.on('ended', this.onAudioEnded.bind(this));
        }
    }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
            var _this3 = this;

            var soundCloudAudio = this.soundCloudAudio;
            var _props2 = this.props;
            var streamUrl = _props2.streamUrl;
            var resolveUrl = _props2.resolveUrl;

            var playedBefore = this.state.playing;

            function restartIfPlayed() {
                if (playedBefore) {
                    soundCloudAudio.play();
                }
            }

            if (streamUrl !== nextProps.streamUrl) {
                soundCloudAudio.stop();
                soundCloudAudio.preload(nextProps.streamUrl);
                restartIfPlayed();
            } else if (resolveUrl !== nextProps.resolveUrl) {
                soundCloudAudio.stop();
                soundCloudAudio.resolve(nextProps.resolveUrl, function (data) {
                    _this3.setState(_defineProperty({}, data.tracks ? 'playlist' : 'track', data));
                    restartIfPlayed();
                });
            }
        }
    }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
            this.soundCloudAudio.unbindAll();
        }
    }, {
        key: 'onSeekingTrack',
        value: function onSeekingTrack() {
            this.setState({ seeking: true });
        }
    }, {
        key: 'onSeekedTrack',
        value: function onSeekedTrack() {
            this.setState({ seeking: false });
        }
    }, {
        key: 'onAudioStarted',
        value: function onAudioStarted() {
            var soundCloudAudio = this.soundCloudAudio;
            var onStartTrack = this.props.onStartTrack;


            this.setState({ playing: true });

            (0, _audioStore.stopAllOther)(soundCloudAudio.playing);
            (0, _audioStore.addToPlayedStore)(soundCloudAudio);

            onStartTrack && onStartTrack(soundCloudAudio, soundCloudAudio.playing);
        }
    }, {
        key: 'onAudioPaused',
        value: function onAudioPaused() {
            var onPauseTrack = this.props.onPauseTrack;

            this.setState({ playing: false });

            onPauseTrack && onPauseTrack(this.soundCloudAudio);
        }
    }, {
        key: 'onAudioEnded',
        value: function onAudioEnded() {
            var onStopTrack = this.props.onStopTrack;

            this.setState({ playing: false });

            onStopTrack && onStopTrack(this.soundCloudAudio);
        }
    }, {
        key: 'getCurrentTime',
        value: function getCurrentTime() {
            this.setState({ currentTime: this.soundCloudAudio.audio.currentTime });
        }
    }, {
        key: 'getDuration',
        value: function getDuration() {
            this.setState({ duration: this.soundCloudAudio.audio.duration });
        }
    }, {
        key: 'wrapChild',
        value: function wrapChild(child) {
            var newProps = (0, _objectAssign2.default)({}, { soundCloudAudio: this.soundCloudAudio }, this.state);
            return _react2.default.cloneElement(child, newProps);
        }
    }, {
        key: 'render',
        value: function render() {
            var children = this.props.children;


            if (!children) {
                return null;
            }

            if (!Array.isArray(children)) {
                var child = children;
                return this.wrapChild(child);
            }

            return _react2.default.createElement(
                'span',
                null,
                _react2.default.Children.map(children, this.wrapChild)
            );
        }
    }]);

    return SoundPlayerContainer;
}(_react.Component);

SoundPlayerContainer.propTypes = {
    streamUrl: _react.PropTypes.string,
    resolveUrl: _react.PropTypes.string,
    clientId: _react.PropTypes.string,
    soundCloudAudio: _react.PropTypes.instanceOf(_soundcloudAudio2.default),
    onStartTrack: _react.PropTypes.func,
    onStopTrack: _react.PropTypes.func,
    onPauseTrack: _react.PropTypes.func
};

exports.default = SoundPlayerContainer;